import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import { UpdateHomePageTestimonialSectionCmsDto } from "./dto/update-home-page-testimonial-section-cms.dto";
import { HomePageTestimonialSectionCmsRepository } from "./home-page-testimonial-section-cms.repository";
import { HomePageTestimonialSectionCmsEntity } from "./entities/home-page-testimonial-section-cms.entity";

@Injectable()
export class HomePageTestimonialSectionCmsService {
    constructor(
        @InjectRepository(HomePageTestimonialSectionCmsRepository)
        private readonly repository: HomePageTestimonialSectionCmsRepository
    ) {
    }

    async updateOrCreateData(
        dto: UpdateHomePageTestimonialSectionCmsDto
    ): Promise<HomePageTestimonialSectionCmsEntity | Error> {
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HomePageTestimonialSectionCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}