import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { PackagesService } from "./packages.service";
import { CreatePackagesDto } from "./dto/create-packages.dto";
import { UpdatePackagesDto } from "./dto/update-packages.dto";

@Controller({ path: "packages", version: Constants.API_VERSION_1 })

export class PackagesController {
    constructor(private readonly service: PackagesService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    create(
        @Body() dto: CreatePackagesDto,
    ) {
        return this.service.create(dto);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    update(
        @Param("id") id: string, 
        @Body() dto: UpdatePackagesDto,
    ) {
        return this.service.update(id, dto);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
