import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateShippingMethodsDto} from "./dto/create-shipping-method.dto";
import {UpdateShippingMethodsDto} from "./dto/update-shipping-method.dto";
import {ShippingMethods} from "./entities/shipping-method.entity";

@EntityRepository(ShippingMethods)
export class ShippingMethodsRepository extends Repository<ShippingMethods> {
    async createEntity(dto: CreateShippingMethodsDto): Promise<ShippingMethods | Error> {
        try {
            const {serviceName, serviceType, price} = dto;
            const entity = this.create({serviceName, serviceType, price});
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }


    async findAll(): Promise<ShippingMethods[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<ShippingMethods | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateShippingMethodsDto): Promise<ShippingMethods | Error> {
        try {
            await this.update(id, {
                serviceName: dto.serviceName,
                serviceType: dto.serviceType,
                price: dto.price
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ShippingMethods[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(ShippingMethods)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}