import { Body, Controller, Delete, Get, Param, Patch, Post, UploadedFiles, UseGuards, UseInterceptors } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { TestimonialsService } from "./testimonials.service";
import { CreateTestimonialsDto } from "./dto/create-testimonials.dto";
import { UpdateTestimonialsDto } from "./dto/update-testimonials.dto";

@Controller({ path: "testimonials", version: Constants.API_VERSION_1 })

export class TestimonialsController {
    constructor(private readonly service: TestimonialsService) {}

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 }
        ]
    ))
    create(
        @Body() dto: CreateTestimonialsDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 }
        ]
    ))
    update(
        @Param("id") id: string, 
        @Body() dto: UpdateTestimonialsDto, 
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
