import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {
    convertBase64ToImage,
    convertBase64ToPDF,
    formattedDateString,
    generateOrderId,
    generateRandomPassword,
} from 'src/helper/common-helper';
import {DirectPost} from 'src/helper/nmi-payment-gatway-helper';
import {CreateListingStatusDto} from 'src/modules/mail-sender/dto/create-listing-status.dto';
import {CreateMailSenderDto} from 'src/modules/mail-sender/dto/create-mail-sender.dto';
import {MailSenderService} from 'src/modules/mail-sender/mail-sender.service';
import {PaymentConfigurationRepository} from 'src/modules/payment-configuration/payment-configuration.repository';
import {PaymentGatewayRepository} from 'src/modules/payment-gateway/payment-gateway.repository';
import {UpsCoreService} from 'src/modules/ups/ups-core/ups-core.service';
import {UpsShippingService} from 'src/modules/ups/ups-shipping/ups-shipping.service';
import {UsersRepository} from 'src/modules/users/user.repository';
import {Constants} from 'src/utils/constants';
import CryptoUtils from 'src/utils/crypto.utils';
import {ResponseUtils} from 'src/utils/response.utils';
import {CouponRepository} from '../coupon/coupon.repository';
import {OrderDetailsRepository} from '../order-details/order-details.repository';
import {CreateOrderByAdminDto} from './dto/create-order-by-admin.dto';
import {CreateOrderDto} from './dto/create-order.dto';
import {UpdateOrderDto} from './dto/update-order.dto';
import {Order} from './entity/orders.entity';
import {OrderRepository} from './order-repository';
import {CreateOrderByGuestDto} from './dto/create-order-by-guest.dto';
import {RefundItemRepository} from 'src/modules/refund-item/refund-item.repository';
import {OrderPlacedDto} from 'src/modules/mail-sender/dto/order-placed-mail.dto';
import {OrderStatusChangeDto} from 'src/modules/mail-sender/dto/order-status-change-mail.dto';
import {ReturnProductsRepository} from 'src/modules/return-products/return-products.repository';
import { ProductsRepository } from 'src/modules/product/product/product.repository';
import { DoSpacesService } from 'src/modules/SpacesModule/SpacesService/doSpacesService';
import { ContactPageCmsRepository } from 'src/modules/CMS/contact-page-cms/contact-page-cms.repository';

@Injectable()
export class OrdersService {
    constructor(
        @InjectRepository(OrderRepository)
        private readonly repository: OrderRepository,
        @InjectRepository(OrderDetailsRepository)
        private readonly orderDetailrepo: OrderDetailsRepository,
        private readonly productRepository: ProductsRepository,
        private readonly couponsRepository: CouponRepository,
        private readonly orderDetailsRepository: OrderDetailsRepository,
        private readonly userRepository: UsersRepository,
        private readonly servMail: MailSenderService,
        private readonly upsShipping: UpsShippingService,
        private readonly upsCore: UpsCoreService,
        @InjectRepository(PaymentConfigurationRepository)
        private readonly paymentConfigurationRepository: PaymentConfigurationRepository,
        @InjectRepository(PaymentGatewayRepository)
        private readonly paymentGatewayRepository: PaymentGatewayRepository,
        @InjectRepository(RefundItemRepository)
        private readonly refundItemRepository: RefundItemRepository,
        @InjectRepository(ContactPageCmsRepository)
        private readonly contactPageCmsRepository : ContactPageCmsRepository,
        @InjectRepository(ReturnProductsRepository)
        private readonly returnProductsRepo: ReturnProductsRepository,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    getItemInfo(products: any, productId: any) {
        let newProductId: any = productId;
        let newProductInfo: any = {};

        products.forEach((productInfo: any) => {
            if (productInfo && productInfo.id === newProductId) {
                newProductInfo = productInfo;
            }
        });
        //console.log(JSON.stringify(newProductInfo));
        return newProductInfo;
    }

    async create(dto: CreateOrderDto, user: any): Promise<any | Error> {
        try {
            let productIds: any = [];

            dto.products.forEach(async (product: any) => {
                productIds.push(product.productId);
            });

            const products = await this.productRepository.getProductsByQuery(
                productIds,
            );
            const orders = await this.repository.findOrderIdEntity();
            const orderId = await generateOrderId(orders);
            let items: any = [];
            let productItems: any = [];
            let subTotal: any = 0;
            let total: any = 0;
            let totalQuantity: any = 0;

            //  loop through the products
            let orderItems: any = dto.products;

            await orderItems.forEach(async (itemInfo: any) => {
                let itemSubTotal: any = 0;
                let itemTotal: any = 0;
                let itemQuantity: any = 0;
                let itemPrice: any = 0;
                let details: any = {};
                let newItemInfo: any = {};

                //console.log("product id: " + itemInfo.productId);
                //Get products
                newItemInfo = this.getItemInfo(products, itemInfo.productId);

                //details of items object populate
                details = {
                    productId: newItemInfo.id,
                    name: newItemInfo.name,
                    customerId: user.id,
                    orderId: orderId,
                    featuredImage: newItemInfo.featuredImage,
                    variants: itemInfo.variants.toString(),
                    quantity: itemInfo.quantity,
                    createdBy: user.id,
                    actualPrice: itemInfo.actualPrice,
                    brandName: newItemInfo.brandName
                };

                // //price addding for both without variants and with variants
                // if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
                //     //Insert Units
                //     let finalPrice = newItemInfo.price;
                //     if (newItemInfo.discountedPrice !== 0 && newItemInfo.discountedPrice < newItemInfo.price) {
                //         finalPrice = newItemInfo.discountedPrice;
                //     }

                //     itemQuantity = itemInfo.quantity;
                //     itemPrice = finalPrice;
                //     itemSubTotal = finalPrice * itemQuantity;
                //     itemTotal = (itemPrice * itemQuantity);

                // }

                if (itemInfo.quantity > 0) totalQuantity += itemInfo.quantity;
                if (itemInfo.itemSubTotal > 0) subTotal += itemInfo.itemSubTotal;
                if (itemInfo.itemSubTotal > 0) total += itemInfo.itemSubTotal;

                details.itemSubTotal = itemInfo.itemSubTotal;
                details.itemTotal = itemInfo.itemSubTotal;
                details.itemPrice = itemInfo.itemPrice;
                // details.discountPrice = itemInfo.discountPrice;
                items.push(details);

                // Product details for email
                let productDetails = details;
                productItems.push(productDetails);
            });

            let couponDiscount = 0;
            if (dto.coupon) {
                couponDiscount =
                    await this.couponsRepository.getCouponDiscountByCouponNumber(
                        dto.coupon,
                        total,
                    );
                if (!couponDiscount) {
                    throw new HttpException(
                        'Coupon is not valid!',
                        HttpStatus.BAD_REQUEST,
                    );
                }
            }

            const placeOrder = {
                orderId: orderId,
                customerId: user.id,
                customerName: user.firstName + user.lastName,
                productPriceTotal: total - couponDiscount,
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount: couponDiscount,
                totalQuantity: totalQuantity,
                itemTotalDiscount: dto.coupon ? dto.coupon : null,
                subTotal: subTotal,
                total: (parseFloat(total) + parseFloat(dto.shippingCost) + parseFloat(dto.salesTaxAmount)),
                remarks: dto.remarks ? dto.remarks : null,
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                shippingAddress: dto.shippingAddress,
                billingAddress: dto.billingAddress,
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: user.id,
                discountType: dto.discountType,
                discountAmount: dto.discountAmount,
                discountTotal: dto.discountTotal,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount: dto.salesTaxAmount,
                cardNumber: `************${dto.ccNum.slice(-4)}`
            };
            const orderPlaced = await this.repository.createEntity(placeOrder);
            // console.log("order placed", orderPlaced);

            items.forEach(async (item: any) => {
                const detailsPlaced = await this.orderDetailsRepository.createEntity(
                    item,
                );
                const productData = await this.productRepository.findOne(
                    item.productId,
                );
            });

            if (orderPlaced) {
                if (dto.paymentMethodName === 'Credit/Debit Card') {
                    const paymentConf = await this.paymentConfigurationRepository.find();
                    if (!paymentConf) {
                        throw new HttpException(
                            'Payment security key not found',
                            HttpStatus.NOT_FOUND,
                        );
                    }
                    const dp = new DirectPost(paymentConf[0].securityKey);
                    const billingData = dto.billingAddress
                        ? JSON.parse(dto.billingAddress)
                        : {};
                    const shippingData = dto.shippingAddress
                        ? JSON.parse(dto.shippingAddress)
                        : {};
                    const billingName = billingData.name.split(' ');
                    const shippingName = shippingData.name.split(' ');
                    const billingInfo = {
                        first_name: billingName[0],
                        last_name: billingName.length > 1 ? billingName[1] : '',
                        address1: billingData.address,
                        city: billingData.city,
                        state: billingData.stateCode,
                        zip: billingData.postCode,
                    };
                    const shippingInfo = {
                        shipping_first_name: shippingName[0],
                        shipping_last_name: shippingName.length > 1 ? shippingName[1] : '',
                        shipping_address1: shippingData.addressOne,
                        shipping_city: shippingData.city,
                        shipping_state: shippingData.stateCode,
                        shipping_zip: shippingData.postCode,
                    };

                    dp.setBilling(billingInfo);
                    dp.setShipping(shippingInfo);
                    const data = await dp.doSale(
                        dto.amount,
                        dto.ccNum,
                        dto.ccExp,
                        dto.cvv,
                    );

                    if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === '100' && data['transactionid']) {
                        const payload = {
                            customerId: user.id,
                            customerName: user.firstName + ' ' + user.lastName,
                            invoiceID: orderId,
                            paymentAmount: dto.amount,
                            paymentDate: new Date(),
                            paymentTransID: data['transactionid'],
                            paymentPayload: JSON.stringify(data),
                            transStatus: 'Success',
                        };
                        const res = await this.paymentGatewayRepository.createEntity(
                            payload,
                        );
                        // console.log("payment gateway response=", res);
                        // update payment status
                        const updateOrder = await this.repository.update(
                            orderPlaced['id'],
                            {paymentStatus: 'Paid'},
                        );
                        // mail to user and admin
                        // let mailAdminDto = {
                        //     to: [user.email],
                        //     from: process.env.EMAIL_SENDER_MAIL,
                        //     subject: 'Order Placed Notification.',
                        //     listingOwner: user.name,
                        //     listingStatus: "Pending",
                        //     listingID: orderId
                        // };
                        // const adminMail = await this.servMail.sendNewListingNotification(
                        //     mailAdminDto as CreateListingStatusDto,
                        // );
                        // console.log(adminMail);
                        const contactCMS = await this.contactPageCmsRepository.findAll();
                        let mailDto = {
                            to: [user.email],
                            from: process.env.EMAIL_SENDER_MAIL,
                            subject: 'Order Placed Notification',
                            customerName: user.firstName + ' ' + user.lastName,
                            orderData: orderPlaced,
                            totalAmount: orderPlaced.total,
                            orderDate: formattedDateString(orderPlaced.createdAt),
                            shippingAddress: JSON.parse(orderPlaced.shippingAddress),
                            contactInfo: contactCMS ? contactCMS[0] : {},
                            items: productItems,
                        };
                        const mail = await this.servMail.sendPlaceOrderNotification(
                            mailDto as OrderPlacedDto,
                        );
                        // console.log('Mail: ', mail);
                        return ResponseUtils.successResponseHandler(
                            200,
                            'Order placed and payment processed successfully.',
                            'orderId',
                            orderId
                        );
                    } else {
                        // delete order
                        const orderDetails = await this.orderDetailsRepository.find({
                            orderId: orderPlaced['orderId'],
                        });
                        if (orderDetails.length > 0) {
                            orderDetails.forEach(async (element: any) => {
                                await this.orderDetailsRepository.remove(element);
                            });
                        }
                        await this.repository.remove(orderPlaced);
                        return ResponseUtils.errorResponseHandler(
                            400,
                            'Payment failed! Please try again',
                            'errors',
                            data,
                        );
                    }
                } else {
                    const contactCMS = await this.contactPageCmsRepository.findAll();
                    let mailDto = {
                        to: [user.email],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: 'Order Placed Notification',
                        customerName: user.firstName + ' ' + user.lastName,
                        orderData: orderPlaced,
                        totalAmount: (parseFloat(orderPlaced.total) + parseFloat(orderPlaced.shippingCost) + parseFloat(orderPlaced.salesTaxAmount)) - parseFloat(orderPlaced.totalDiscount),
                        orderDate: formattedDateString(orderPlaced.createdAt),
                        shippingAddress: JSON.parse(orderPlaced.shippingAddress),
                        contactInfo: contactCMS ? contactCMS[0] : {},
                        items: productItems,
                    };
                    // console.log('Mail Dto: ', mailDto);
                    const mail = await this.servMail.sendPlaceOrderNotification(
                        mailDto as OrderPlacedDto,
                    );
                    // console.log('Mail: ', mail);
                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order placed successfully.',
                        'orderId',
                        orderId
                    );
                }
            } else {
                return ResponseUtils.errorResponseHandler(
                    400,
                    'Failed to create order data!',
                );
            }
        } catch (e) {
            console.log(e);
            throw e;
        }
    }

    async findAll(
        take: number,
        page: number,
        paymentStatus?: string,
        deliveryStatus?: string,
        startDate?: Date,
        endDate?: Date,
        customerName?: string,
        orderId?: string,
    ): Promise<Order[] | Error> {
        let query: any = {};
        if (paymentStatus) query.paymentStatus = paymentStatus;
        if (deliveryStatus) query.deliveryStatus = deliveryStatus;
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (customerName) query.customerName = customerName;
        if (orderId) query.orderId = orderId;

        const data = await this.repository.findAllEntities(take, page, query);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Orders data successfully fetched.',
            'data',
            data,
        );
    }

    async findOrderByFilter(
        authUser: any,
        limit: number,
        page: number,
        tabStatus: string,
    ): Promise<any | Error> {
        if (!authUser) {
            throw new HttpException(Constants.UNAUTH_REQ, HttpStatus.FORBIDDEN);
        }

        // if (tabStatus === 'dashboard') {
        //     const totalOrdersList = await this.repository.totalUserOrderList(limit, page, authUser.user.id);
        //     const totalOrders = await this.repository.countUserOrder(authUser.user.id);
        //     const totalDeliveredOrders = await this.repository.countUserDeliveredOrder(authUser.user.id, 'DELIVERED');
        //     const totalCancelledOrders = await this.repository.countUserCancelledOrder(authUser.user.id, 'CANCELLED');
        //     const totalShippedOrders = await this.repository.countUserShippedOrder(authUser.user.id, 'SHIPPED');
        //     const totalPendingOrders = await this.repository.countUserPendingOrder(authUser.user.id, 'PENDING');

        //     const responseData = {
        //         totalOrdersList: totalOrdersList,
        //         totalOrders: totalOrders,
        //         totalDeliveredOrders: totalDeliveredOrders,
        //         totalCancelledOrders: totalCancelledOrders,
        //         totalShippedOrders: totalShippedOrders,
        //         totalPendingOrders: totalPendingOrders
        //     }

        //     return responseData;
        // }

        if (tabStatus === 'totalOrders') {
            return await this.repository.totalUserOrderList(
                limit,
                page,
                authUser.user.id,
            );
        }

        if (tabStatus === 'pending') {
            return await this.repository.totalUserPendingOrderList(
                limit,
                page,
                authUser.user.id,
                'PENDING',
            );
        }

        if (tabStatus === 'shipped') {
            return await this.repository.totalUserShippedOrderList(
                limit,
                page,
                authUser.user.id,
                'SHIPPED',
            );
        }

        if (tabStatus === 'delivered') {
            return await this.repository.totalUserDeliveredOrderList(
                limit,
                page,
                authUser.user.id,
                'DELIVERED',
            );
        }

        if (tabStatus === 'cancelled') {
            return await this.repository.totalUserCancelledOrderList(
                limit,
                page,
                authUser.user.id,
                'CANCELLED',
            );
        }

        if (tabStatus === 'processing') {
            return await this.repository.totalUserProcessingOrderList(
                limit,
                page,
                authUser.user.id,
                'PROCESSING',
            );
        }
    }

    async orderProfitService(
        take: number,
        page: number,
        startDate?: Date,
        endDate?: Date,
        orderId?: string,
    ): Promise<Order[] | Error> {
        let query: any = {};
        if (startDate) query.startDate = startDate;
        if (endDate) query.endDate = endDate;
        if (orderId) query.orderId = orderId;

        const data = await this.orderDetailrepo.findAllEntities(take, page, query);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }
        return ResponseUtils.successResponseHandler(
            200,
            'Orders data successfully fetched.',
            'data',
            data,
        );
    }

    async findSeparatOrderByFilter(
        limit: number,
        page: number,
        orderId?: string,
        customerId?: string,
    ): Promise<any | Error> {
        const data = await this.repository.findSearchedOrder(
            limit,
            page,
            orderId,
            customerId,
        );
        const totalOrders = await this.repository.countUserOrder(customerId);
        const totalDeliveredOrders = await this.repository.countUserDeliveredOrder(
            customerId,
            'Delivered',
        );
        const totalCancelledOrders = await this.repository.countUserCancelledOrder(
            customerId,
            'Cancelled',
        );
        const totalShippedOrders = await this.repository.countUserShippedOrder(
            customerId,
            'Shipped',
        );

        const responseData = {
            data: data,
            totalOrders: totalOrders,
            totalDeliveredOrders: totalDeliveredOrders,
            totalCancelledOrders: totalCancelledOrders,
            totalShippedOrders: totalShippedOrders,
        };

        return responseData;
    }

    async findOne(id: string): Promise<Order | Error> {
        const data = await this.repository.findOneEntityWihoutRelation(id);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        //console.log('data = ', data);
        //return data;

        const paymentGateway = await this.paymentGatewayRepository.findOne({
            where: {
                invoiceID:id,
                isRefunded: true,
                isDeleted: false,
            },
        });
        if (paymentGateway) {
            if (
                parseFloat(paymentGateway.paymentAmount) == paymentGateway.refundAmount
            ) {
                data[0]['isGlobalRefunded'] = true;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            } else {
                data[0]['isGlobalRefunded'] = false;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            }
        } else {
            data[0]['isGlobalRefunded'] = false;
            data[0]['refundAmount'] = 0;
        }

        const orderDetailsData = await this.orderDetailsRepository.findAllEntityByOrderID(id);
        //return orderDetailsData;
        data[0]['orders'] = orderDetailsData;
        //return data;
        for (const order of orderDetailsData) {
            const { orderId, productId } = order;
            
            const returnItems = await this.returnProductsRepo.find({
                where: {
                    orderId,
                    productId,
                    isDeleted: false,
                },
            });

            let returnQuantity = 0;
            let returnStatus = '';

            for (const item of returnItems) {
                if (item.status === 'Approved' || item.status == 'Unapproved') {
                    returnQuantity += item.quantity;
                    returnStatus = item.status;
                }
            }

            const refundItem = await this.refundItemRepository.checkOrderIsRefunded(orderId, productId);

            const orderIndex = data[0]['orders'].findIndex(o => o.orderId === orderId && o.productId === productId);
            if(orderIndex !== -1) {
                data[0]['orders'][orderIndex]['isRefunded'] = !!refundItem;
                data[0]['orders'][orderIndex]['returnQuantity'] = returnQuantity;
                data[0]['orders'][orderIndex]['returnStatus'] = returnStatus;
            }
        }

        return ResponseUtils.successResponseHandler(
            200,
            'Order data successfully fetched',
            'Orders',
            data,
        );
    }
    
    async findOneOrderByOrderID(id: string): Promise<any | Error> {
        const data = await this.repository.findOneEntityWihoutRelation(id);
        if (!data) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        //console.log('data = ', data);
        //return data;

        const paymentGateway = await this.paymentGatewayRepository.findOne({
            where: {
                invoiceID:id,
                isRefunded: true,
                isDeleted: false,
            },
        });
        if (paymentGateway) {
            if (
                parseFloat(paymentGateway.paymentAmount) == paymentGateway.refundAmount
            ) {
                data[0]['isGlobalRefunded'] = true;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            } else {
                data[0]['isGlobalRefunded'] = false;
                data[0]['refundAmount'] = paymentGateway.refundAmount;
            }
        } else {
            data[0]['isGlobalRefunded'] = false;
            data[0]['refundAmount'] = 0;
        }

        const orderDetailsData = await this.orderDetailsRepository.findAllEntityByOrderID(id);
        //return orderDetailsData;
        data[0]['orders'] = orderDetailsData;
        //return data;
        for (const order of orderDetailsData) {
            const { orderId, productId } = order;
            
            const returnItems = await this.returnProductsRepo.find({
                where: {
                    orderId,
                    productId,
                    isDeleted: false,
                },
            });

            let returnQuantity = 0;
            let returnStatus = '';

            for (const item of returnItems) {
                if (item.status === 'Approved' || item.status == 'Unapproved') {
                    returnQuantity += item.quantity;
                    returnStatus = item.status;
                }
            }

            const refundItem = await this.refundItemRepository.checkOrderIsRefunded(orderId, productId);

            const orderIndex = data[0]['orders'].findIndex(o => o.orderId === orderId && o.productId === productId);
            if(orderIndex !== -1) {
                data[0]['orders'][orderIndex]['isRefunded'] = !!refundItem;
                data[0]['orders'][orderIndex]['returnQuantity'] = returnQuantity;
                data[0]['orders'][orderIndex]['returnStatus'] = returnStatus;
            }
        }

        return ResponseUtils.successResponseHandler(
            200,
            'Order data successfully fetched',
            'Orders',
            data,
        );
    }

    async remove(id: string): Promise<boolean | Error> {
        const deleteOrder = await this.repository.deleteEntity(id);
        if (!deleteOrder) {
            throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
        }

        const deleteOrderDetails = await this.orderDetailsRepository.deleteEntity(
            id,
        );

        if (deleteOrder && deleteOrderDetails) {
            return ResponseUtils.buildDeletedData(true);
        } else {
            return ResponseUtils.buildDeletedData(false);
        }
    }

    async update(id: string, dto: UpdateOrderDto): Promise<Order | Error> {
        try {
            const order = await this.repository.findOne(id);
            if (!order) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            const data = await this.repository.updateOrder(id, dto);
            if (!data) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            // Create payment log
            if (dto.paymentStatus === 'Paid') {
                const payload = {
                    customerId: order.id,
                    customerName: order.customerName,
                    invoiceID: order.orderId,
                    paymentAmount: Number(order.subTotal) + Number(order.salesTaxAmount) + Number(order.shippingCost),
                    paymentDate: new Date(),
                    transStatus: 'Success',
                };
                const res = await this.paymentGatewayRepository.createEntity(payload);
                // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });
              
            }

            // if (dto.status === 'PROCESSING') {
            //   const shipAddress = JSON.parse(order.shippingAddress);
            //   if (order.shippingType === 'fedex') {
            //     const shippingDto = {
            //       shippingAddress: {
            //         addressOne: shipAddress.addressOne,
            //         addressTwo: shipAddress.addressTwo,
            //         city: shipAddress.city,
            //         stateCode: shipAddress.stateCode,
            //         postCode: shipAddress.postCode,
            //         countryCode: shipAddress.countryCode,
            //       },
            //       serviceType: order.serviceType,
            //       weightValue: order.totalWeight,
            //       labelStockType: dto.labelStockType,
            //       labelImageType: dto.labelImageType
            //     };
            //     const fedexShipping = await this.fedexShipping.create(shippingDto);
            //     if (fedexShipping['output']) {
            //       await this.repository.update(id, {
            //         trackingNumber:
            //           fedexShipping['output'].transactionShipments[0]
            //             .masterTrackingNumber,
            //         labelStockType: dto.labelStockType,
            //         labelImageType: dto.labelImageType,
            //         labelImageEncode: fedexShipping['output'].pieceResponses[0].packageDocuments[0].encodedLabel
            //       });
            //     }
            //   }
            //   if (order.shippingType === 'ups') {
            //     const dto = {
            //       shippingAddress: {
            //         name: shipAddress.name,
            //         mobile: shipAddress.mobile,
            //         addressOne: shipAddress.addressOne,
            //         addressTwo: shipAddress.addressTwo,
            //         city: shipAddress.city,
            //         stateCode: shipAddress.stateCode,
            //         postCode: shipAddress.postCode,
            //         countryCode: shipAddress.countryCode,
            //       },
            //       serviceType: order.serviceType,
            //       weightValue: order.totalWeight,
            //     };
            //     const upsShipping = await this.upsShipping.create(dto);
            //     if (upsShipping['ShipmentResponse']) {
            //       await this.repository.update(id, {
            //         trackingNumber:
            //           upsShipping['ShipmentResponse'].ShipmentResults.PackageResults
            //             .TrackingNumber,
            //       });
            //     }
            //   }
            // }

            if (order['status'] != dto.status) {
                const customerEmail = await this.userRepository.findOne(
                    data['customerId'],
                );

                // send mail
                let proGiftItems: any = [];
                // fetch order details
                const orderDetails = await this.orderDetailsRepository.find({
                    where: {
                        orderId: order.orderId,
                        isDeleted: false
                    }
                });

                for (let i = 0; i < Object.keys(orderDetails).length; i++) {
                    let data = {
                        name: orderDetails[i].name,
                        quantity: orderDetails[i].quantity,
                        itemPrice: orderDetails[i].itemPrice,
                        featuredImage: orderDetails[i].featuredImage
                    };
                    proGiftItems.push(data);
                }
                // console.log('details: ', orderDetails);

                const contactCMS = await this.contactPageCmsRepository.findAll();
                const userEmail = customerEmail['email'];
                let mailDto = {
                    to: [userEmail],
                    from: process.env.EMAIL_SENDER_MAIL,
                    subject: 'Order Status Changed Notification',
                    customerName: data['customerName'],
                    orderData: data,
                    totalAmount: parseFloat(data["total"]),
                    orderDate: formattedDateString(data['createdAt']),
                    shippingAddress: JSON.parse(data['shippingAddress']),
                    contactInfo: contactCMS ? contactCMS[0] : {},
                    items: proGiftItems
                };;
                const mail = await this.servMail.sendOrderStatusChangeNotification(
                    mailDto as unknown as OrderStatusChangeDto,
                );
            }
            return ResponseUtils.successResponseHandler(
                200,
                'Order data updated!',
                'data',
                data,
            );
        } catch (err) {
            console.log(err);
            throw err;
        }
    }

    async updateStripeRefund(id: string, dto: UpdateOrderDto): Promise<Order | Error> {
        try {
            const order = await this.repository.findOne(id);
            if (!order) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            const data = await this.repository.updateOrder(id, dto);
            if (!data) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            return ResponseUtils.successResponseHandler(
                200,
                'Order data updated!',
                'data',
                data,
            );
        } catch (err) {
            console.log(err);
            throw err;
        }
    }

    async validateShipping(dto: any): Promise<any | Error> {
        try {
            let data: any;
            const order = await this.repository.findOne(dto.id);
            if (!order) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            let orderId = order.id;
            const shipAddress = JSON.parse(order.shippingAddress);
            if (order.shippingType === 'fedex') {
                const payload = {
                    shippingAddress: {
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postalCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                    labelStockType: order.labelStockType,
                    labelImageType: order.labelImageType
                };
               
                if (data['output']) {
                    await this.repository.updateOrderStatus(orderId, 'SHIPPED');
                }
            }

            if (order.shippingType === 'ups') {
                const payload = {
                    shippingAddress: {
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postalCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                };
                
                if (
                    data['PickupCreationResponse'].Response.ResponseStatus.Description ===
                    'Success'
                ) {
                    await this.repository.updateOrderStatus(orderId, 'SHIPPED');
                }
            }
            return ResponseUtils.successResponseHandler(200, '', 'data', data);
        } catch (err) {
            console.log(err);
            throw err;
        }
    }

    async updateOrderTracking(dto: any): Promise<any | Error> {
        try {
            let data: any;

            const order = await this.repository.findOne({
                where: {
                    orderId: dto.id,
                },
            });
            if (!order) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            let orderId = order.id;

            await this.repository.update(orderId, {
                trackingNumber: dto.trackingNumber,
            });

            const datas = await this.repository.findOne({
                where: {
                    orderId: dto.id,
                },
            });
            if (!datas) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            return ResponseUtils.successResponseHandler(
                200,
                'Order info updated',
                'data',
                datas,
            );
        } catch (err) {
            console.log(err);
            throw err;
        }
    }

    async createOrderByAdmin(
        dto: CreateOrderByAdminDto,
        user: any,
    ): Promise<Order | Error> {
        try {
            let productIds: any = [];
            let orderFor: any;

            dto.products.forEach(async (product: any) => {
                productIds.push(product.productId);
            });

            orderFor = await this.userRepository.findOne({
                where: {email: dto.email},
            });

            const products = await this.productRepository.getProductsByQuery(
                productIds,
            );
            // const orders = await this.repository.findOrderIdEntity();
            // const orderId = await generateOrderId(orders);
            const orderId = dto.orderId;
            let items: any = [];
            let subTotal: any = 0;
            let total: any = 0;
            let totalQuantity: any = 0;

            //  loop through the products
            let orderItems: any = dto.products;

            if (!orderFor) {
                const generateRandomPass = generateRandomPassword(8);
                const userDto = {
                    email: dto.email,
                    firstName: dto.customerName,
                    lastName: null,
                    city: null,
                    zipCode: null,
                    country: null,
                    phone: null,
                    image: null,
                    company: null,
                    userType: 'customer',
                    password: await CryptoUtils.encrypt(generateRandomPass),
                    discountType: dto.discountType,
                    discountAmount: dto.discountAmount,
                };
                const createUser = await this.userRepository.createEntity(userDto);
                orderFor = createUser;
                if (orderFor) {
                    // let mailDto = {
                    //     "to": [
                    //         dto.email
                    //     ],
                    //     "from": process.env.EMAIL_SENDER_MAIL,
                    //     "subject": "HGS account creation notification.",
                    //     "username": dto.customerName,
                    //     "password": generateRandomPass
                    // }
                    // await this.servMail.send(mailDto as CreateMailSenderDto);
                    const contactCMS = await this.contactPageCmsRepository.findAll();
                    let mailDto = {
                        to: [dto.email],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: 'Ganja Container User Account Creation Notification',
                        username: dto.customerName,
                        password: generateRandomPass,
                        contactInfo: contactCMS ? contactCMS[0] : {},    
                    };

                    const mail = await this.servMail.accocuntCreateByAdminCustomerNotify(
                        mailDto as CreateMailSenderDto,
                    );
                }
            } else if (orderFor && orderFor.isDeleted === true) {
                const data = await this.userRepository.update(orderFor.id, {
                    isDeleted: false,
                });
            }

            await orderItems.forEach(async (itemInfo: any) => {
                let itemSubTotal: any = 0;
                let itemTotal: any = 0;
                let itemQuantity: any = 0;
                let itemPrice: any = 0;
                let details: any = {};
                let newItemInfo: any = {};

                //Get products
                newItemInfo = this.getItemInfo(products, itemInfo.productId);

                //details of items object populate
                details = {
                    productId: newItemInfo.id,
                    name: newItemInfo.name,
                    customerId: orderFor.id,
                    orderId: orderId,
                    featuredImage: newItemInfo.featuredImage,
                    variants: itemInfo.variants.toString(),
                    quantity: itemInfo.quantity,
                    createdBy: user.id,
                    actualPrice: itemInfo.actualPrice,
                    brandName: newItemInfo.brandName
                };

                // //price addding for both without variants and with variants
                // if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
                //     //Insert Units
                //     let finalPrice = newItemInfo.price;
                //     if (newItemInfo.discountedPrice !== 0 && newItemInfo.discountedPrice < newItemInfo.price) {
                //         finalPrice = newItemInfo.discountedPrice;
                //     }

                //     itemQuantity = itemInfo.quantity;
                //     itemPrice = finalPrice;
                //     itemSubTotal = finalPrice * itemQuantity;
                //     itemTotal = (itemPrice * itemQuantity);

                // }

                if (itemInfo.quantity > 0) totalQuantity += itemInfo.quantity;
                if (itemInfo.itemSubTotal > 0) subTotal += itemInfo.itemSubTotal;
                if (itemInfo.itemSubTotal > 0) total += itemInfo.itemSubTotal;

                details.itemSubTotal = itemInfo.itemSubTotal;
                details.itemTotal = itemInfo.itemSubTotal;
                details.itemPrice = itemInfo.itemPrice;
                items.push(details);
            });

            let couponDiscount = 0;
            if (dto.coupon) {
                couponDiscount =
                    await this.couponsRepository.getCouponDiscountByCouponNumber(
                        dto.coupon,
                        total,
                    );
                if (!couponDiscount) {
                    throw new HttpException(
                        'coupon is not valid',
                        HttpStatus.BAD_REQUEST,
                    );
                }
            }

            //let lastName

            const placeOrder = {
                orderId: orderId,
                customerId: orderFor.id,
                customerName:
                    orderFor.firstName +
                    ' ' +
                    (orderFor.lastName ? orderFor.lastName : ''),
                productPriceTotal: total - couponDiscount,
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount: couponDiscount,
                totalQuantity: totalQuantity,
                itemTotalDiscount: dto.coupon ? dto.coupon : null,
                subTotal: dto.itemSubTotal ? dto.itemSubTotal : subTotal,
                total: dto.itemTotal ? dto.itemTotal : total,
                remarks: dto.remarks ? dto.remarks : null,
                shippingAddress: dto.shippingAddress,
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: user.id,
                discountType: dto.discountType,
                discountAmount: dto.discountAmount,
                discountTotal: dto.discountTotal,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount: dto.salesTaxAmount,
                totalDiscount: dto.totalDiscount,
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                paymentStatus: dto.paymentStatus,
                cardNumber: `************${dto.ccNum.slice(-4)}`
            };
            const orderPlaced = await this.repository.createEntity(placeOrder);

            items.forEach(async (item: any) => {
                const detailsPlaced = await this.orderDetailsRepository.createEntity(
                    item,
                );
                const productData = await this.productRepository.findOne(
                    item.productId,
                );
            });

            if (orderPlaced) {
                if (dto.paymentMethodName === 'Credit/Debit Card') {
                    const paymentConf = await this.paymentConfigurationRepository.find();
                    if (!paymentConf) {
                        throw new HttpException(
                            'Payment security key not found',
                            HttpStatus.NOT_FOUND,
                        );
                    }
                    const dp = new DirectPost(paymentConf[0].securityKey);
                    const billingData = dto.shippingAddress
                        ? JSON.parse(dto.shippingAddress)
                        : {};
                    const billingName = billingData.name;
                    const billingInfo = {
                        first_name: billingName,
                        address1: billingData.addressOne,
                        city: billingData.city,
                        state: billingData.stateCode,
                        zip: billingData.postCode,
                    };
                    dp.setBilling(billingInfo);
                    const data = await dp.doSale(
                        dto.amount,
                        dto.ccNum,
                        dto.ccExp,
                        dto.cvv,
                    );

                    if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === '100' && data['transactionid']) {
                        const payload = {
                            customerId: orderFor.id,
                            customerName: orderFor.firstName + ' ' + orderFor.lastName,
                            invoiceID: orderId,
                            paymentAmount: dto.amount,
                            paymentDate: new Date(),
                            paymentTransID: data['transactionid'],
                            paymentPayload: JSON.stringify(data),
                            transStatus: 'SUCCESS',
                        };
                        const res = await this.paymentGatewayRepository.createEntity(
                            payload,
                        );
                        // const updateOrder = await this.orderRepository.update(orderPlaced['id'], { paymentStatus: 'PAID' });
                        let mailAdminDto = {
                            to: [orderFor.email],
                            from: process.env.EMAIL_SENDER_MAIL,
                            subject: 'Order Placed Notification.',
                            customerName: user.firstName + ' ' + user.lastName,
                            orderData: orderPlaced,
                            totalAmount: parseFloat(orderPlaced.total),
                            listingOwner: user.fullName,
                            listingStatus: 'Pending',
                            listingID: orderId,
                            shippingAddress: JSON.parse(orderPlaced.shippingAddress),
                            items: items,
                        };
                        const adminMail = await this.servMail.sendNewListingNotification(
                            mailAdminDto as CreateListingStatusDto,
                        );
                        return ResponseUtils.successResponseHandler(
                            200,
                            'Order Placed and Payment successful',
                            'orderId',
                            orderId
                        );
                    } else {
                        await this.repository.hardDeleteEntity(
                            orderPlaced.id,
                            orderPlaced.orderId,
                        );
                        return ResponseUtils.errorResponseHandler(
                            400,
                            'Payment failed! Please try again',
                            'errors',
                            data,
                        );
                    }
                } else {
                    let mailAdminDto = {
                        to: [orderFor.email],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: 'Order Placed Notification.',
                        customerName: user.firstName + ' ' + user.lastName,
                        orderData: orderPlaced,
                        totalAmount: parseFloat(orderPlaced.total),
                        listingOwner: user.fullName,
                        listingStatus: 'Pending',
                        listingID: orderId,
                        shippingAddress: JSON.parse(orderPlaced.shippingAddress),
                        items: items,
                    };
                    const adminMail = await this.servMail.sendNewListingNotification(
                        mailAdminDto as CreateListingStatusDto,
                    );
                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order created successfully',
                        'orderId',
                        orderId
                    );
                }
            } else {
                return ResponseUtils.errorResponseHandler(
                    400,
                    'Failed to create order data!',
                );
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }

    async createOrderByGuest(dto: CreateOrderByGuestDto): Promise<any | Error> {
        try {
            let productIds: any = [];
            const guestId = '9999999999999';
            const guestName = 'Guest';

            dto.products.forEach(async (product: any) => {
                productIds.push(product.productId);
            });

            const products = await this.productRepository.getProductsByQuery(
                productIds,
            );

            const orders = await this.repository.findOrderIdEntity();
            const orderId = await generateOrderId(orders);
            let items: any = [];
            let subTotal: any = 0;
            let total: any = 0;
            let totalQuantity: any = 0;

            // Loop through the products
            let orderItems: any = dto.products;
            await orderItems.forEach(async (itemInfo: any) => {
                let itemSubTotal: any = 0;
                let itemTotal: any = 0;
                let itemQuantity: any = 0;
                let itemPrice: any = 0;
                let details: any = {};
                let newItemInfo: any = {};
                newItemInfo = this.getItemInfo(products, itemInfo.productId);

                // Details of items object populate
                details = {
                    productId: newItemInfo.id,
                    name: newItemInfo.name,
                    customerId: guestId,
                    orderId: orderId,
                    featuredImage: newItemInfo.featuredImage,
                    variants: itemInfo.variants.toString(),
                    quantity: itemInfo.quantity,
                    createdBy: guestId,
                    brandName: newItemInfo.brandName
                };

                // Price addding for both without variants and with variants
                if (itemInfo && itemInfo.productId && itemInfo.quantity > 0) {
                    let finalPrice = newItemInfo.price;
                    if (
                        newItemInfo.discountedPrice !== 0 &&
                        newItemInfo.discountedPrice < newItemInfo.price
                    ) {
                        finalPrice = newItemInfo.discountedPrice;
                    }
                    itemQuantity = itemInfo.quantity;
                    itemPrice = finalPrice;
                    itemSubTotal = finalPrice * itemQuantity;
                    itemTotal = itemPrice * itemQuantity;
                }

                if (itemQuantity > 0) totalQuantity += itemQuantity;
                if (itemSubTotal > 0) subTotal += itemSubTotal;
                if (itemTotal > 0) total += itemTotal;
                details.itemSubTotal = itemSubTotal;
                details.itemTotal = itemTotal;
                details.itemPrice = itemPrice;
                items.push(details);
            });

            // Coupon discount
            let couponDiscount = 0;
            if (dto.coupon) {
                couponDiscount =
                    await this.couponsRepository.getCouponDiscountByCouponNumber(
                        dto.coupon,
                        total,
                    );
                if (!couponDiscount) {
                    throw new HttpException(
                        'Coupon is not valid!',
                        HttpStatus.BAD_REQUEST,
                    );
                }
            }

            const placeOrder = {
                orderId: orderId,
                customerId: guestId,
                customerName: guestName,
                productPriceTotal: total,
                totalQuantity: totalQuantity,
                subTotal: subTotal,
                total: total,
                remarks: dto.remarks ? dto.remarks : null,
                paymentMethodName: dto.paymentMethodName ? dto.paymentMethodName : null,
                shippingAddress: dto.shippingAddress,
                billingAddress: dto.billingAddress,
                shippingCost: dto.shippingCost,
                shippingType: dto.shippingType,
                serviceType: dto.serviceType,
                totalWeight: dto.weightValue,
                createdBy: guestId,
                couponNumber: dto.coupon ? dto.coupon : null,
                couponDiscount: couponDiscount,
                salesTaxTitle: dto.salesTaxTitle,
                salesTaxRate: dto.salesTaxRate,
                salesTaxAmount: dto.salesTaxAmount,
                cardNumber: `************${dto.ccNum.slice(-4)}`
            };
            const orderPlaced = await this.repository.createEntity(placeOrder);
            items.forEach(async (item: any) => {
                const detailsPlaced = await this.orderDetailsRepository.createEntity(
                    item,
                );
                const productData = await this.productRepository.findOne(
                    item.productId,
                );
            });
            if (orderPlaced) {
                if (dto.paymentMethodName === 'Credit/Debit Card') {
                    const paymentConf = await this.paymentConfigurationRepository.find();
                    if (!paymentConf) {
                        throw new HttpException(
                            'Payment security key not found!',
                            HttpStatus.NOT_FOUND,
                        );
                    }
                    const dp = new DirectPost(paymentConf[0].securityKey);
                    const billingData = dto.billingAddress
                        ? JSON.parse(dto.billingAddress)
                        : {};
                    const shippingData = dto.shippingAddress
                        ? JSON.parse(dto.shippingAddress)
                        : {};
                    const billingName = billingData.name.split(' ');
                    const shippingName = shippingData.name.split(' ');
                    const billingInfo = {
                        first_name: billingName[0],
                        last_name: billingName.length > 1 ? billingName[1] : '',
                        address1: billingData.address,
                        city: billingData.city,
                        state: billingData.stateCode,
                        zip: billingData.postCode,
                    };
                    const shippingInfo = {
                        shipping_first_name: shippingName[0],
                        shipping_last_name: shippingName.length > 1 ? shippingName[1] : '',
                        shipping_address1: shippingData.addressOne,
                        shipping_city: shippingData.city,
                        shipping_state: shippingData.stateCode,
                        shipping_zip: shippingData.postCode,
                    };
                    dp.setBilling(billingInfo);
                    dp.setShipping(shippingInfo);
                    const data = await dp.doSale(
                        dto.amount,
                        dto.ccNum,
                        dto.ccExp,
                        dto.cvv,
                    );
                    if (data['statusCode'] === 200 && data['response'] === "1" && data['response_code'] === '100' && data['transactionid']) {
                        const payload = {
                            customerId: guestId,
                            customerName: guestName,
                            invoiceID: orderId,
                            paymentAmount: dto.amount,
                            paymentDate: new Date(),
                            paymentTransID: data['transactionid'],
                            paymentPayload: JSON.stringify(data),
                            transStatus: 'Success',
                        };
                        const res = await this.paymentGatewayRepository.createEntity(
                            payload,
                        );
                        // Update payment status
                        const updateOrder = await this.repository.update(
                            orderPlaced['id'],
                            {paymentStatus: 'Paid'},
                        );
                        // Mail to user and admin
                        const billingData = dto.billingAddress
                            ? JSON.parse(dto.billingAddress)
                            : {};
                        const shippingData = dto.shippingAddress
                            ? JSON.parse(dto.shippingAddress)
                            : {};
                        let mailAdminDto = {
                            to: [billingData.email, shippingData.email],
                            from: process.env.EMAIL_SENDER_MAIL,
                            subject: 'Order Placed Notification.',
                            listingOwner: billingData.name,
                            listingStatus: 'Pending',
                            listingID: orderId,
                        };
                        const adminMail = await this.servMail.sendNewListingNotification(
                            mailAdminDto as CreateListingStatusDto,
                        );
                        return ResponseUtils.successResponseHandler(
                            200,
                            'Order placed and payment successfully completed.',
                            'orderId',
                            orderId
                        );
                    } else {
                        // Delete order
                        const orderDetails = await this.orderDetailsRepository.find({
                            orderId: orderPlaced['orderId'],
                        });
                        if (orderDetails.length > 0) {
                            orderDetails.forEach(async (element: any) => {
                                await this.orderDetailsRepository.remove(element);
                            });
                        }
                        await this.repository.remove(orderPlaced);
                        return ResponseUtils.errorResponseHandler(
                            400,
                            'Payment failed! Please try again.',
                            'errors',
                            data,
                        );
                    }
                } else {
                    const billingData = dto.billingAddress
                        ? JSON.parse(dto.billingAddress)
                        : {};
                    const shippingData = dto.shippingAddress
                        ? JSON.parse(dto.shippingAddress)
                        : {};
                    let mailAdminDto = {
                        to: [billingData.email, shippingData.email],
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: 'Order Placed Notification.',
                        listingOwner: billingData.name,
                        listingStatus: 'Pending',
                        listingID: orderId,
                    };
                    const adminMail = await this.servMail.sendNewListingNotification(
                        mailAdminDto as CreateListingStatusDto,
                    );
                    return ResponseUtils.successResponseHandler(
                        200,
                        'Order created successfully.',
                        'orderId',
                        orderId
                    );
                }
            } else {
                return ResponseUtils.errorResponseHandler(
                    400,
                    'Failed to create order data!',
                );
            }
        } catch (e) {
            console.log(e);
            throw e;
        }
    }

    async createStripeOrderByGuest(dto: any): Promise<any | Error> {
        try {
            //return dto;
            //const productsCart=JSON.parse(dto.checkoutStripeData.products);

            const productsCart=JSON.parse(dto.checkoutStripeData.products);
            const items: any[] = [];
            let subTotal = 0;
            let total = 0;
            let totalQuantity = 0;
            for (const itemInfo of productsCart) {
                const productDetail=itemInfo;
                const finalPrice = Math.min(
                    parseFloat(productDetail?.discountPrice) > 0 ? parseFloat(productDetail?.discountPrice) : Infinity,
                    parseFloat(productDetail?.tierWiseUnitPrice) > 0 ? parseFloat(productDetail?.tierWiseUnitPrice) : Infinity,
                    parseFloat(productDetail?.price) || Infinity
                );
                const itemQuantity = itemInfo.quantity;
                const itemPrice = finalPrice;
                const itemSubTotal = itemPrice * itemQuantity;
                const itemTotal = itemSubTotal;
                totalQuantity += itemQuantity;
                subTotal += itemSubTotal;
                total += itemTotal;
                const details = {
                    productId: productDetail.id,
                    name: productDetail.name,
                    customerId: dto.orderData.customerId,
                    orderId: dto.checkoutStripeData.orderID,
                    featuredImage: productDetail.featuredImage,
                    variants: (itemInfo?.variants?.toString() || ''),
                    quantity: itemInfo.quantity,
                    createdBy: dto.orderData.customerId,
                    brandName: productDetail.brandName,
                    itemSubTotal: itemSubTotal,
                    itemTotal: itemTotal,
                    itemPrice: itemPrice,
                };
                items.push(details);
            }

            // const detailsPlaced = await this.orderDetailsRepository.createEntity(items[0]);
            // return detailsPlaced;
            //return items;
            //return productsCart;
            const orderCount=await this.repository.checkOrderCount(dto.checkoutStripeData.orderID);

            
            if(orderCount==0)
            {
                const placeOrder = {
                    orderId: dto.checkoutStripeData.orderID,
                    customerId: dto.orderData.customerId,
                    customerName: dto.orderData.customerName,
                    productPriceTotal: dto.orderData.productPriceTotal,
                    totalQuantity: dto.orderData.totalQuantity,
                    subTotal: dto.orderData.subTotal,
                    total: dto.orderData.total,
                    remarks: null,
                    paymentMethodName: dto.orderData.paymentMethodName,
                    shippingAddress: JSON.stringify(dto.shipping),
                    billingAddress: JSON.stringify(dto.billing),
                    shippingCost: dto.orderData.shippingCost,
                    shippingType: dto.orderData.shippingType,
                    serviceType: dto.orderData.serviceType,
                    totalWeight: dto.orderData.weightValue,
                    createdBy: dto.orderData.customerId,
                    couponNumber: null,
                    couponDiscount: 0.00,
                    salesTaxTitle: dto.orderData.salesTaxTitle,
                    salesTaxRate: dto.orderData.salesTaxRate,
                    salesTaxAmount: dto.orderData.salesTaxAmount,
                    cardNumber: `************`,
                    paymentStatus:'Paid',
                };
                const orderPlaced = await this.repository.createEntity(placeOrder);
                //console.log('After Order Create Data: ', orderPlaced);
                console.log('orderPlaced =', orderPlaced);
                
                //return orderPlaced
                if(orderPlaced)
                {
                    items.forEach(async (item: any) => {
                        const detailsPlaced = await this.orderDetailsRepository.createEntity(item);
                        //return detailsPlaced;
                        if(detailsPlaced)
                        {
                            const productData = await this.productRepository.findOne(item.productId);
                          
                        }
                    });

                    const paymentPayload = dto.paymentPayload;
                    paymentPayload.invoiceID=dto.checkoutStripeData.orderID;
                    const res = await this.paymentGatewayRepository.createEntity(paymentPayload);
                    
                    //console.log('orderPlaced =', orderPlaced);
                    let mailDto = {
                        to:dto.billing.email,
                        from: process.env.EMAIL_SENDER_MAIL,
                        subject: 'Order Placed Notification',
                        customerName: dto.paymentPayload?.customerName,
                        orderData: orderPlaced,
                        totalAmount: orderPlaced.total,
                        orderDate: formattedDateString(orderPlaced.createdAt),
                        shippingAddress: JSON.parse(orderPlaced.shippingAddress),
                        contactInfo: {phone: "+34454523453"},
                        items: items,
                    };
                    
                    const mail = await this.servMail.sendPlaceOrderNotification(
                        mailDto as any,
                    );
                    return true;
                }
                else
                {
                    return true;
                }
            }
            else
            {
                //const orderCount=await this.repository.checkOrderCount(dto.checkoutStripeData.hgsOrderID);
                //console.log('orderCount =', orderCount);
                return true;
            }
            
        } catch (e) {
            console.log(e);
           //throw e;
           return false;
        }
    }

    async getOrderId(): Promise<any | Error> {
        const orders = await this.repository.findOrderIdEntity();
        const orderId = await generateOrderId(orders);
        return orderId;
    }

    async checkOrderId(orderId: string): Promise<any | Error> {
        const data = await this.repository.findOne({
            where: {
                orderId: orderId,
                isDeleted: false,
            },
        });

        if (data) {
            return true;
        } else {
            return false;
        }
    }

    async createShippingLabel(id: string, dto: any): Promise<any | Error> {
        try {
            const order = await this.repository.findOne(id);
            if (!order) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            let orderShippingType = "ups";

            // update order table
            await this.repository.update(id, {
                labelStockType: dto.labelStockType,
                labelImageType: dto.labelImageType
            });
            const data = await this.repository.findOne(id);
            if (!data) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }

            const shipAddress = JSON.parse(order.shippingAddress);
            if (orderShippingType === 'fedex') {
                const shippingDto = {
                    shippingAddress: {
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                    labelStockType: dto.labelStockType,
                    labelImageType: dto.labelImageType
                };
               
            } else if (orderShippingType === 'ups') {
                console.log('orderShippingType = ', orderShippingType);
                const dto = {
                    shippingAddress: {
                        name: shipAddress.name,
                        mobile: shipAddress.mobile,
                        addressOne: shipAddress.addressOne,
                        addressTwo: shipAddress.addressTwo,
                        city: shipAddress.city,
                        stateCode: shipAddress.stateCode,
                        postCode: shipAddress.postCode,
                        countryCode: shipAddress.countryCode,
                    },
                    serviceType: order.serviceType,
                    weightValue: order.totalWeight,
                };
                const upsShipping = await this.upsShipping.create(dto);
                console.log('upsShipping = ', upsShipping);
                console.log('upsShipping = ', upsShipping?.data?.response?.errors);
                console.log('upsShipping = ', upsShipping?.ShipmentResponse?.ShipmentResults?.PackageResults);
                if (upsShipping?.ShipmentResponse?.ShipmentResults?.PackageResults) {
                    const upsData = upsShipping?.ShipmentResponse?.ShipmentResults?.PackageResults;
                    if (upsData?.ShippingLabel?.ImageFormat?.Code === 'PNG') {
                        const base64ToImage = convertBase64ToImage(upsData?.ShippingLabel?.GraphicImage);
                        const image: any = await this.doSpacesService.uploadFileBuffer(base64ToImage, `${upsData?.ShippingLabel?.ImageFormat?.Code.toLowerCase()}`, "uploads/FedExLabelImage/");
                        console.log('image =', image);
                        await this.repository.update(id, {
                            trackingNumber:
                            upsData?.TrackingNumber,
                            labelImageEncode: image
                        });
                    }

                } else {
                    return ResponseUtils.errorResponseHandler(
                        upsShipping?.Response?.ResponseStatus?.Code,
                        'Shipping Label Create Failed!',
                        'data',
                        upsShipping,
                    );
                }

            }
            return ResponseUtils.successResponseHandler(
                200,
                'Order data updated!',
                'data',
                data,
            );
        } catch (err) {
            console.log(err);
            return err;
        }
    }
}
