import { EntityRepository, getConnection, getManager, Repository } from "typeorm";
import { ProductOfferVariants } from "./entities/product-offer-variants.entity";
import { CreateProductOfferVariantsDto } from "./dto/create-product-offer-variants.dto";
import { UpdateProductOfferVariantsDto } from "./dto/update-product-offer-variants.dto";

@EntityRepository(ProductOfferVariants)
export class ProductOfferVariantsRepository extends Repository<ProductOfferVariants> {

    async createEntity(dto: CreateProductOfferVariantsDto): Promise<ProductOfferVariants | Error> {
        try {
            const { productId, productName, offerId, offerTitle } = dto;
            const entity = this.create({
                productId,
                productName,
                offerId,
                offerTitle
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductOfferVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteOfferVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductOfferVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertOfferVariant(offer: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductOfferVariants)
                .values(offer)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductOfferVariant(id: any, offerEntity: any): Promise<any | Error> {
        try {
            const deleteCurrentData = await this.deleteOfferVariant(id);
            await this.bulkInsertOfferVariant(offerEntity)
        } catch (e) {
            return e;
        }
    }

    async findOffersByProductId(productId: string): Promise<ProductOfferVariants[] | Error> {
        try {
            const sql = `SELECT product_offer_variants.*, product_offers.title as offerTitle, product_offers.image as offerImage
            FROM product_offer_variants LEFT JOIN product_offers
            ON product_offer_variants.offerId = product_offers.id WHERE product_offer_variants.productId = '${productId}';`;
            const entityManager = getManager();
            return entityManager.query(sql);
        } catch (e) {
            return e;
        }
    }
}