import { Module } from '@nestjs/common';
import { ReturnProductsService } from './return-products.service';
import { ReturnProductsController } from './return-products.controller';
import { JwtModule } from '@nestjs/jwt';
import JwtConfigService from 'src/core/jwt/jwt-config.service';
import { TypeOrmModule } from '@nestjs/typeorm';
import { UsersRepository } from '../users/user.repository';
import { ReturnProductsRepository } from './return-products.repository';
import JwtHelper from 'src/core/jwt/jwt.helper';
import { PaymentConfigurationRepository } from '../payment-configuration/payment-configuration.repository';
import { PaymentGatewayRepository } from '../payment-gateway/payment-gateway.repository';
import { RefundItemRepository } from '../refund-item/refund-item.repository';
import { OrderRepository } from '../orders/order/order-repository';
import { ProductsRepository } from '../product/product/product.repository';
import { StripeModule } from '../orders/stripe/stripe.module';

@Module({
    imports: [
        JwtModule.registerAsync({
            useClass: JwtConfigService,
        }),
        TypeOrmModule.forFeature([
            UsersRepository,
            ReturnProductsRepository,
            PaymentConfigurationRepository,
            PaymentGatewayRepository,
            RefundItemRepository,
            ProductsRepository,
            OrderRepository
        ]),
        StripeModule
    ],
    controllers: [ReturnProductsController],
    providers: [ReturnProductsService, JwtHelper],
})
export class ReturnProductsModule {  }
