import { Controller, Get, Param, Query, Req, UseGuards } from '@nestjs/common';
import { Constants } from "src/utils/constants";
import { SiteFrontendService } from "./site-frontend.service";
import { AuthGuard } from 'src/core/guards/auth.guard';
import { AdminAuthGuard } from 'src/core/guards/admin-auth.guard';

@Controller({
    path: "site",
    version: Constants.API_VERSION_1
})
export class SiteFrontendController {
    constructor(private readonly service: SiteFrontendService) { }

    @Get('/about-page')
    async getAboutPageData() {
        return await this.service.getAboutPageData();
    }

    @Get('/faq-page')
    async getFaqPageData() {
        return await this.service.getFaqPageData();
    }

    @Get('/contact-page')
    async getContactPageData() {
        return await this.service.getContactPageData();
    }

    @Get('/blog-list')
    getBlogListData(
        @Query("page") page: number,
        @Query("limit") limit: number
    ) {
        return this.service.getBlogListData(page, limit);
    }

    @Get('/blog-details/:urlSlug')
    getBlogDetailsData(@Param('urlSlug') urlSlug: string) {
        return this.service.getBlogDetailsData(urlSlug);
    }

    @Get('/home-page')
    async getHomePageData() {
        return await this.service.getHomePageData();
    }

    @Get('/header-footer')
    async getHeaderFooterData() {
        return await this.service.getHeaderFooterData();
    }

    @UseGuards(AuthGuard)
    @Get("/getUserAddresses")
    getUserAddresses(
        @Req() request: any
    ) {
        return this.service.getUserAddresses(request.user.id);
    }

    @UseGuards(AdminAuthGuard)
    @Get("/getUserAddresses/:userId")
    getUserAddressesByAdmin(@Param('userId') userId: string) {
        return this.service.getUserAddresses(userId);
    }

    @Get('/products')
    async findAllProducts(
        @Query('limit') limit: any,
        @Query('page') page: any
    ) {
        return await this.service.findAllProducts(limit, page);
    }

    @Get('/products/:slug')
    async findOneBySlug(@Param('slug') slug: string) {
        return await this.service.findOneBySlug(slug);
    }

    @Get('/sales-tax-with-coupon')
    async getSalesTaxAndCouponData() {
        return await this.service.getSalesTaxAndCouponData();
    }
    
}
