import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import {FileFieldsInterceptor} from '@nestjs/platform-express';
import {Constants} from 'src/utils/constants';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {TaxExemptService} from './tax-exempt.service';
import {CreateTaxExemptDto} from './dto/create-tax-exempt.dto';
import {UpdateTaxExemptDto} from './dto/update-tax-exempt.dto';
import { UploadedMulterFileI } from '../SpacesModule/SpacesService';


@Controller({path: 'tax-exempt', version: Constants.API_VERSION_1})

export class TaxExemptController {
    constructor(private readonly service: TaxExemptService) {
    }

    @Post()
    // @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'taxFile', maxCount: 1}
    ]))
    create(
        @Body() dto: CreateTaxExemptDto,
        @UploadedFiles() files: {
            taxFile?: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get('/unapproved')
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get('/approved')
    @UseGuards(AdminAuthGuard)
    findApprovedList() {
        return this.service.findApprovedList();
    }

    @Get('/rejected')
    @UseGuards(AdminAuthGuard)
    findRejectedList() {
        return this.service.findRejectedList();
    }

    @Get('/user/:id')
    findByUserId(@Param('id') id: string) {
        return this.service.findByUserId(id);
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch('/approve-user/:id')
    @UseGuards(AdminAuthGuard)
    approveUser(@Param('id') id: string) {
        return this.service.approveUser(id);
    }

    @Patch('/reject-user/:id')
    @UseGuards(AdminAuthGuard)
    rejectUser(@Param('id') id: string) {
        return this.service.rejectUser(id);
    }

    @Patch('/:id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'taxFile', maxCount: 1}
    ]))
    update(
        @Param('id') id: string,
        @Body() dto: UpdateTaxExemptDto,
        @UploadedFiles() files: {
            taxFile?: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
